
package Rcam; # perl's idea of a library module, here it's a class, too.
use warnings;
use strict;
#Object::Tiny does the boilerplate for an object.  The qw stuff gets accessors, only needed for debugging here.
use Object::Tiny qw{
 host_name
 cmd_path
 media_base_path
 local
log_callback
};
use Filesys::SmbClient; # for talking to remote hosts
use File::Copy; # only works well for local since it stats the files w/o knowing some are remote.
use Sys::Hostname; # for knowing if a host is remote or not (makes bareword hostname into the current host's name, weird)

#@@@ edited  9/19/16 to change die to loggit on lines:
# 124, 128, 129, 163 187
#


#@@@ assumptions! (I use the @@@ to make searching for dodgy stuff easier)
# the machine running this has set up /var/lib/FusorFiles with subdirs of the cam machine names with which
# this will be used, and they're writeable by "anybody"
# all vid cap machines are setup so the cam software is at /var/www/html with the otherwise normal structure
# and FIFO in that dir is world writeable.
# end_box.sh (our code, in /var/www/html/macros on each cam machine) makes the cap.txt file under 
# /var/www/html/media once the vid is ready for copy.
# We wound up stat()ing the cap.txt file, so no fixed delays are required
# But always beware the code that isn't there as it won't need to be.

#@@@ yes, this is research-grade code, tightend up neither for extra "my vars" (garbage collection) nor
# security - note the pi's on samba are in the MOUNTAIN workgroup, and the password is hardcoded as dotdot.
# The machine names are also hardcoded.
# Your mileage had better vary!

#@@@ @@@ Worse yet, and quite nasty from a design POV - I use a subroutine from main:: in this!
# It's called loggit(), needs parens, and I'm using it instead of die to report what went wrong, as actually
# dieing most of the time is not a good thing when controlling this stuff.
# The reasoning (if any) is that only main::loggit() knows how to talk to the log window in GTK3
# as well as the log file and we *do* want to know about issues but not necessarily die messy.

# cap.txt in media_base_path names the last video when boxing is done (some time after stop, we'll have to wait a bit)

# host_name is used as a folder name under /var/lib/FusorFiles to copy captured video to on this machine.

# cmd_path is to wherever the pi's FIFO is for starting and stopping vid cap via writing 'ca 1' to start, 'ca 0' to stop.

# We strip off vi_number_  the the pi adds before the rest of the vid name as it's a meaningless sequence number for us,
# and that filename is already ugly-long (should sort better without it, too).

my $debug = 0; # class wide

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#@@@ GAAAAACCCCCCKKKK @@@ Don't do this!  I'm calling back into main:: because only it has some context
# to make logging work as desired.  Kind of circular...perl lets me do this?  No wonder people freak out.
# Yet, it works and is quite expedient.
 #main::loggit(shift); # just pass string/arg to the function in main

#But I don't do it that way anymore, this is closer to the right way, but requres self to be passed in even from "self", yuck
sub loggit { # I'd rather log than die most of the time (make that decision elsewhere/optional)
 my $self = shift; # weird but we pass this to ourself
 $self->{log_callback}->(shift);
}

##############################################################################
sub init() { # make sure samba and network are ok, build paths for later on
my $self = shift;
die "No log callback provided for logging!:$$!" unless $self->{log_callback};
loggit $self, "Rcam: Can't connect to a null hostname!" unless $self->{'host_name'} ; 
# check if we want to do this to "ourself", the rest of the code is different (no need or desire for SMB in that case)
$self->{local} = '0'; # in case some bozo sets it in constructor
my $host = hostname; # ask module where we are running - weird bareword use from it
# debug print "host: $host passed: $self->{'host_name'}\n";
if ($host =~ m/$self->{host_name}/i) # we are local
{
# print "matched\n";
 $self->{local} = 1; # tell ourself the status
 $self->{cmd_path} = '/var/www/html/FIFO'; # assumes our standard setup for file locations in pi cam
 $self->{media_base_path} = '/var/www/html/media/';
#@@@ I could stat FIFO here but if it's not on the local machine, how are we even running at all?
}
 else { # we need samba

#@@@ does not hiccup until actual use even with bullshit params
 $self->{smb} = new Filesys::SmbClient(username => "pi",password => "dotdot", workgroup => "MOUNTAIN"); # be pi
#note this works fine with a bullshit host name passed in, as we don't actually connect to it yet
 loggit ($self,"Rcam:$self->{'host_name'}: Couldn't connect to network $!.") unless $self->{smb};
 $self->{cmd_path} = 'smb://' . $self->{host_name} . '/cam/FIFO'; # go ahead and build these strings
 $self->{media_base_path} = 'smb://' . $self->{host_name} . '/media/'; # when it isn't time-critical
  unless ($self->{smb}->stat( $self->{cmd_path})) {
  loggit $self,"Couldn't connect to command path $self->{cmd_path}: $!";
  return 0; # fail warning
  }
 }
# implied true return if any of it worked
#loggit($self, "create $self->{host_name} worked")
}
##############################################################################
sub start_cap() { # begin recording video at target machine
my $self = shift;
my $fd;
 my $msg;

if ($self->{local}) {
# has to be tested on a pi that has a cam
   open($fd,">", "$self->{cmd_path}")  or loggit $self, "couldn't open FIFO  to start at local $self->{hostname} $!";
   print $fd "ca 1";
   close $fd or return 0; # fail
 } else { # use smb
  $msg = $self->{media_base_path};
  $fd = $self->{smb}->open(">" . "$self->{cmd_path}")  or loggit $self, "couldn't open FIFO to start cap at smb $self->{hostname} $!";
  $self->{smb}->write($fd, "ca 1") or loggit $self,"couldn't write FIFO at $self->{hostname} $!";  #ca 1 is the command to capture
  $self->{smb}->close($fd);
  $self->{smb}->unlink("$self->{media_base_path}" . 'cap.txt') or loggit $self,"couldn't delete cap.txt at $msg"; # fail but continue
 }
 return 1; # assume success
}
##############################################################################
sub end_cap() { # stop recording on target machine, begin boxing
my $self = shift;
my $fd;
if ($self->{local}) {
   open($fd,">", "$self->{cmd_path}")  or loggit $self,"couldn't open FIFO to  end cap $! at $self=>(cmdpath)";
   print $fd "ca 0";
   close $fd;
 } else { # use smb
  $fd = $self->{smb}->open(">" . "$self->{cmd_path}")  or loggit $self, "couldn't open FIFO $! at $self=>(cmdpath)";
  $self->{smb}->write($fd, "ca 0") or loggit $self,"couldn't write FIFO to stop record at $self->{hostname} $!";  #ca 0 is the command to stop
  $self->{smb}->close($fd);
 }
}
##############################################################################
sub copy_vid() { # get video file post-boxing...@@@timing?
# destination path /var/lib/FusorFiles/host_name/filename
my $self = shift;
my $sfd;
my $readdat;
my @stattab; # fresh null one for each call to this - a good use of garbage collection
my $sourcepath = $self->{media_base_path}; 
my $rawsource;
my $smbdir;
my $destination = '/var/lib/FusorFiles/' . "$self->{host_name}" . '/'; # all but vid filename
my $timeout; # variable for seeing how long we're waiting

if ($self->{local}) {
  # do local copy here
# Delay loop here till @table = $self->{smb}=>stat( "$self->{media_base_path}" . 'cap.txt');
# The idea is to make it more idiot resistant.  A smart idiot just won't call this too soon!
 until (defined $stattab[7]) {
  @stattab = stat("$self->{media_base_path}" . 'cap.txt');
  print "Waiting for: $self->{media_base_path}" . 'cap.txt' ."\n" if $debug;
  loggit $self,"waiting for:$self->{media_base_path}" . 'cap.txt';
  sleep 1; # no point beating this too hard - the boxing process is slow anyway
 }
   open($sfd, "<", "$self->{media_base_path}" . 'cap.txt');
   $readdat = <$sfd>;
   close $sfd;
   chomp $readdat;
   $sourcepath .= $readdat; # path to source video
   print "sourcepath: $sourcepath\n" if ($debug);
   $readdat =~ s/vi_\d+_//; # remove leading v_sequence#_
   $destination .= $readdat;
   print "destination: $destination\n" if ($debug);
   copy($sourcepath,$destination) or loggit $self,"vid copy failed: $! : $sourcepath";
   return $destination;
 } else { # use smb to do it

# Delay loop here till @table = $self->{smb}=>stat( "$self->{media_base_path}" . 'cap.txt');
# The idea is to make it more idiot resistant.  A smart idiot just won't call this too soon!
 $timeout = 0; 
until (defined $stattab[7]) {
  @stattab = $self->{smb}->stat("$self->{media_base_path}" . 'cap.txt');
  print "Waiting for: $self->{media_base_path}" . 'cap.txt' ."\n" if $debug;
  loggit $self,"waiting for:$self->{media_base_path}" . 'cap.txt';
  sleep 1; # no point beating this too hard - the boxing process is slow anyway
  $timeout++;
  if ($timeout > 180) # pick a number, 180 is 3 min and should be painful enough
  {
   loggit $self,"timed out, file not copied";
   return;  # no point hanging forever if slave failed
  }
 }
# now get auto-generated filename from target pi cam
  local *FD;
  tie(*FD,'Filesys::SmbClient');
  open(FD,'<' . "$self->{media_base_path}" . 'cap.txt');
  $readdat = <FD>;
  close (FD); # be clean
  chomp $readdat;
  $rawsource = $readdat;
  $readdat =~ s/vi_\d+_//; # remove leading v_sequence#_
  $destination .= $readdat;
  print "destination: $destination\n" if ($debug);
  $smbdir = $self->{media_base_path}; # could put this before delay loop above to save a ms.
  $smbdir =~ s/smb://; # redundant and error with smbclient process
  open($sfd, "| smbclient  $smbdir dotdot > /dev/null 2>&1\n") or loggit $self,"open smbclient $smbdir didn't work for copy $!\n";
  loggit $self,"starting copy...";
  print $sfd "get $rawsource $destination\n"; # tell smbclient what we want
  print $sfd "exit\n"; # and quit whenever you're done
  return $destination;
 } # smb copy
}

1; # the usual return true from modules/classes

__END__
# keep dead code to frighten the live stuff
# earlier:
#  $sourcepath .= $readdat; # full path to source video
#  print "sourcepath: $sourcepath\n" if ($debug);

#maybe write my own copy and use fstat from SmbClient?
# warns but works.
#   tie(*FD,'Filesys::SmbClient');
#  open(FD,'<' . "$sourcepath") or die "couldn't open $sourcepath:$!";
#   copy(*FD,"$destination") or die "copy failed: $!";
#   close(FD); 

